/*
    tstore - an archive format for archiving tar archives
    Copyright (C) 2004  Kasper Dupont

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    with permission from the author any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#define _XOPEN_SOURCE 500
#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include <fcntl.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <stdint.h>
#include <string.h>
#include <assert.h>

int size_in_blocks(int fd)
{
  struct stat stbuf;
  fstat(fd,&stbuf);
  if (stbuf.st_size % 512) return 0;
  return stbuf.st_size/512;
}

int main(int argc, char ** argv)
{
  uint32_t store_size;
  int store_fd;
  uint8_t buf[512];
  uint32_t idx,nidx;

  if (argc != 2) {
    fprintf(stderr,"Uage: %s <store>\n",argv[0]);
    return 1;
  }

  store_fd=open(argv[1],O_RDONLY);
  if (store_fd==-1) {
    perror(argv[1]);
    return 1;
  }

  store_size=size_in_blocks(store_fd);
  if (store_size) {
    switch (pread(store_fd,buf,512,0)) {
    case -1:
      perror(argv[1]);
      return 1;
    default:
      store_size=0;
    case 512:
      if (memcmp(buf,"TSTORE1",7)) store_size=0;
    }
  }
  if (!store_size) {
    fprintf(stderr,"%s: Not a valid store\n",argv[1]);
    return 1;
  }

  for (idx=store_size-1; idx; idx=nidx) {
    switch(pread(store_fd,buf,512,idx*512)) {
    case -1:
      perror(argv[1]);
      return 1;
    default:
      fprintf(stderr,"%s: short pread\n",argv[1]);
      return 1;
    case 512:
      break;
    }
    if (buf[16]==0) {
      printf("<Empty slot>\n");
    } else {
      uint64_t size;
      memcpy(&size,buf+8,8);
      printf("%lld\t%s\n",size,buf+16);
    }
    memcpy(&nidx,buf,4);
    if (nidx>=idx) {
      fprintf(stderr,"Invalid link %d=>%d\n",idx,nidx);
      return 1;
    }
  }
  return 0;
}
